# Nombres de paquetes

## Introducción

El código Go es organizado en paquetes. Dentro de un paquete, el código puede hacer referencia a cualquier identificador —nombre— que se haya definido; sin embargo, los usuarios del paquete únicamente pueden hacer referencia a los tipos, funciones, constantes y variables exportadas del paquete. Dichas referencias siempre incluyen el nombre del paquete como prefijo: `foo.Bar` se refiere al nombre exportado `Bar` en el paquete importado llamado `foo`.

La selección de un muy buen nombre de paquete mejora la estructura del código. El nombre de un paquete proporciona un contexto para su contenido, lo que facilita a los usuarios comprender para qué sirve el paquete y cómo utilizarlo. El nombre también facilita a los responsables del paquete determinar qué debe y qué no debe incluirse en él a medida que evoluciona. Los paquetes bien denominados facilitan la búsqueda del código necesario.

[TODO: Encontrar la forma de como relacionar ambos artículos.]: #

Effective Go proporciona directrices para nombrar paquetes, tipos, funciones y variables. Este artículo amplía esa discusión y analiza los nombres que se encuentran en la biblioteca estándar. También discute los nombres de paquetes inadecuados y cómo corregirlos.

## Nombres de paquetes

Los nombres de paquetes deben ser cortos y claros. Se escriben en minúsculas, sin guiones bajos[^§] ni alternar mayúsculas y minúsculas[^†]. Suelen ser sustantivos simples. El estilo de nombres típico de otro lenguaje puede no ser idiomático en un programa Go. Aquí hay dos ejemplos de nombres que podrían ser de una buena práctica en otros lenguajes, no obstante no se ajustan bien en Go:

- `computeServiceClient`
- `priority_queue`

Un paquete Go puede exportar varios tipos y funciones. Por ejemplo, un paquete `compute` podría exportar un tipo `Client` con métodos para usar el servicio, así como funciones para dividir una tarea de cálculo entre varios clientes.

**Abrevie con criterio.** Los nombres de los paquetes pueden abreviarse cuando la abreviatura resulte familiar al programador. Los paquetes más utilizados suelen tener nombres abreviados:

- `strconv` (conversión de cadena)
- `syscall` (llamada al sistema)
- `fmt` (entrada/salida formateada)

Por otro lado, si el abreviar el nombre de un paquete lo hace ambiguo o poco claro, es preferible que no lo haga.

**No plagie nombres al usuario.** Evite dar a un paquete un nombre que se utilice habitualmente en el código del usuario. Por ejemplo, el paquete de E/S con buffer se llama `bufio`, no `buf`, ya que `buf` es un buen nombre de variable para un buffer.

## Denominación del contenido de los paquetes

El nombre de un paquete y los nombres de su contenido están asociados, puesto que el código cliente los emplea conjuntamente. Cuando diseñe un paquete, adopte el punto de vista del usuario.

**Evite la repetición.** Dado que el código cliente usa el nombre del paquete como prefijo cuando se refiere a los contenidos del paquete, los nombres de esos contenidos no necesitan repetir el nombre del paquete. El servidor HTTP proporcionado por el paquete `http` se llama `Server`, no `HTTPServer`. El código cliente se refiere a este tipo como `http.Server`, para que no haya ambigüedad.

[TODO: Incluir traducción del artículo «Package names» https://web.archive.org/web/20230619135210/https://go.dev/blog/package-names.]: #
---

Cuando se importa un paquete, el nombre del paquete se convierte en el elemento de acceso a su contenido. Por ejemplo:

```golang
import "bytes"
```

Por lo tanto, después de importar el paquete en cuestión es posible referirse al mismo cómo `bytes.Buffer`.

Cabe destacar que resulta ser de suma utilidad que se pueda usar el nombre del paquete para referirse a su contenido, lo cual implica que el nombre debe ser ideal, es decir: *corto, conciso y sugerente*. Por convención, los paquetes se nombran en minúsculas, empleando una única palabra; no debería existir la necesidad de incluir guiones bajos[^§] o alternar mayúsculas y minúsculas[^†]. Procure ser breve, ya que todos los que utilicen su paquete escribirán dicho nombre. A priori, no tendría por qué preocuparse por las colisiones entre nombres. El nombre del paquete solo es el identificador por defecto para las importaciones; por lo tanto, no necesita ser único o exclusivo para todo el código fuente, y en el raro caso de que se presente una colisión, el paquete importado puede tomar un nombre diferente para referirse localmente. En cualquier caso, la confusión es rara porque el nombre del archivo en la importación determina exactamente qué paquete se está usando.

Otra convención es que el nombre del paquete corresponde al nombre base del directorio que contiene su código fuente; el paquete en `src/encoding/base64` se importa como, `"encoding/base64"`, pero tiene por nombre `base64`,  y en ningún caso `encoding_base64` o bien `encodingBase64`.

El importador de un paquete empleará el nombre para referirse a su contenido, por lo que los identificadores que son exportables desde el paquete pueden valerse de este hecho para evitar repetirse[^1]. Por ejemplo, el tipo *buffered reader* del paquete `bufio` se llama `Reader`, no `BufReader`, porque los usuarios lo notan como `bufio.Reader`, el cual es un identificador claro y conciso. Además, como las entidades importadas siempre se designan con el nombre del paquete del que proceden, `bufio.Reader` no entra en conflicto con `io.Reader`. Del mismo modo, la función para crear nuevas instancias de `ring.Ring` —el cual es la definición de un *constructor* en Go— se llamaría normalmente `NewRing`. Sin embargo, como `Ring` es el único tipo exportado por dicho paquete, y dado que el paquete se llama `ring`, se le denomina simplemente como `New`, ya que los usuarios del paquete lo podrán notar como `ring.New`. Utilice la estructura del paquete para elegir buenos identificadores.

Otro ejemplo breve es `once.Do`; `once.Do(setup)` se comprende bien y no mejoraría si fuera nombrado como `once.DoOrWaitUntilDone(setup)`. Los nombres largos no mejoran automáticamente la legibilidad. Un comentario puede resultar ser de mayor utilidad en comparación de un nombre extralargo.

## Referencias

- [The Go Blog — Package names][package-names].
- [Effective Go — Package names][effective_go-package_names].

[^§]: Convención de nomenclatura denominada como [Snake case][snake_case].

[^†]: Convención de nomenclatura denominada como [Camel case][camel_case] —ya sea en *UpperCamelCase* (más conocido como «PascalCase») o *lowerCamelCase* (conocido simplemente como «camelCase»)—.

[^1]: No utilice la notación `import .`, la cual puede simplificar las suites de pruebas que necesitan ejecutarse fuera del paquete que se esté probando, por lo que de cualquier otro modo debe evitarse.

[snake_case]: https://es.wikipedia.org/wiki/Snake_case
[camel_case]: https://es.wikipedia.org/wiki/Camel_case
[package-names]: https://web.archive.org/web/20230619135210/https://go.dev/blog/package-names
[effective_go-package_names]: https://web.archive.org/web/20230606092234/https://go.dev/doc/effective_go#names
